@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: webindex.c 355 2025-05-30 23:19:48Z leres $ (XSE)
+ * @(#) $Id: webindex.c 361 2025-09-09 18:23:54Z leres $ (XSE)
  *
  * Copyright (c) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2013, 2015, 2016, 2017, 2018, 2019, 2022, 2023, 2025
  *	Craig Leres
@@ -70,6 +70,10 @@ static const char copyright[] =
 /* Convert value to K */
 #define C2K(k) (((k) + 1023) / 1024)
 
+/* Convert tm to year/month/day */
+#define TM2YMD(tm) \
+    (((tm->tm_year + 1900) * 10000) + (tm->tm_mon * 100) + tm->tm_mday)
+
 /* Is this webfile a directory? */
 #define WEBFILE_ISDIR(wp) ((wp)->w_type == T_DIR || (wp)->w_type == T_PARENT)
 #define WEBFILE_ISINVISIBLE(wp) (((wp)->w_flags & W_INVISIBLE) != 0)
@@ -182,7 +186,9 @@ static struct s2v suffix2type[] = {
 void checkmode(const char *, mode_t, int);
 int countfiles(const char *dname, struct cf *);
 int createthumb(struct webfile *, const char *, struct cf *);
+void daysection(FILE *, struct tm *);
 char *findsection(struct webfile *, struct cf *);
+void fmtdaysectionts(struct tm *, char *, size_t);
 void header(FILE *, struct cf *);
 void hookupthumbnails(struct cf *, struct webfile *, int);
 int invisible(const char *, struct cf *, int);
@@ -287,13 +293,15 @@ process(const char *dname)
 	const char *p, *indexfn;
 	const char *table, *tr, *tail;
 	char *cp, *section, *prefixend;
-	int i, row, col, force, intable;
+	char autosection;
+	int i, row, col, force, intable, lastymd, ymd;
 	struct timespec file_tim, ind_tim;
 	struct webfile *wp, *webfiles;
 	struct cf *cf;
 	struct section *sp;
 	size_t cc, nfiles, nwebfiles;
 	struct timespec mtim, ctim;
+	struct tm *tm;
 	char *otz;
 	int d, d2;
 	DIR *dirp;
@@ -631,13 +639,21 @@ process(const char *dname)
 	col = 0;
 	row = 0;
 	intable = 0;
+	lastymd = 0;
 	for (wp = webfiles, i = 0; i < nwebfiles; ++i, ++wp) {
 		if (WEBFILE_ISINVISIBLE(wp))
 			continue;
 
 		/* New row logic */
+		tm = localtime(&wp->w_timestamp);
+		ymd = TM2YMD(tm);
+		autosection =
+		    ((cf->c_flags & C_DAY_SECTION) != 0 && ymd != lastymd);
+
+
 		section = findsection(wp, cf);
-		if (intable && (col >= cf->c_maxcol || section != NULL)) {
+		if (intable && (col >= cf->c_maxcol ||
+		    autosection || section != NULL)) {
 			/* Output last cell (which may span columns) */
 			if (col < cf->c_maxcol && row > 0) {
 				if (cf->c_maxcol - col > 1)
@@ -649,7 +665,8 @@ process(const char *dname)
 			fprintf(f, "</tr>\n");
 
 			/* Avoid huge tables (which take forever to render) */
-			if (++row > cf->c_maxrow || section != NULL) {
+			if (++row > cf->c_maxrow ||
+			    autosection || section != NULL) {
 				fprintf(f, "</table>\n");
 				row = 0;
 				intable = 0;
@@ -657,6 +674,10 @@ process(const char *dname)
 				fprintf(f, "%s", tr);
 			col = 0;
 		}
+		if (autosection) {
+			daysection(f, tm);
+			lastymd = ymd;
+		}
 		if (section != NULL) {
 			fprintf(f, "%s", section);
 			free(section);
@@ -1269,6 +1290,40 @@ nextfile:
 	return (force);
 }
 
+void
+fmtdaysectionts(struct tm *tm, char *buf, size_t len)
+{
+	const char *nth;
+	char fmt[64];
+
+	// https://stackoverflow.com/a/46139480
+	if  (3 < tm->tm_mday && tm->tm_mday < 21) {
+		nth = "th";
+	} else {
+		switch (tm->tm_mday % 10) {
+
+		case 1:
+		    nth = "st";
+		    break;
+
+		case 2:
+		    nth = "nd";
+		    break;
+
+		case 3:
+		    nth = "rd";
+		    break;
+
+
+		default:
+		    nth = "th";
+		    break;
+		}
+	}
+	(void)snprintf(fmt, sizeof(fmt), "%%A, %%B %%e%s, %%Y", nth);
+	(void)strftime(buf, len, fmt, tm);
+}
+
 void
 header(FILE *f, struct cf *cf)
 {
@@ -1697,6 +1752,15 @@ naildir(struct cf *cf)
 	return (".nai");
 }
 
+void
+daysection(FILE *f, struct tm *tm)
+{
+	char tstr[64];
+
+	fmtdaysectionts(tm, tstr, sizeof(tstr));
+	fprintf(f, "<hr>\n<h3>\n%s\n</h3>\n", tstr);
+}
+
 /*
  * Find the section that goes with this filename.
  * The section data is removed from the list and freed.
